@@ -3,6 +3,7 @@ Content related Django models.
 """
 from gettext import gettext as _
 
+import asyncio
 import datetime
 import json
 import tempfile
@@ -721,7 +722,7 @@ class SigningService(BaseModel):
     pubkey_fingerprint = models.TextField()
     script = models.TextField()
 
-    def sign(self, filename):
+    def sign(self, filename, env_vars=None):
         """
         Signs the file provided via 'filename' by invoking an external script (or executable).
 
@@ -731,6 +732,7 @@ class SigningService(BaseModel):
 
         Args:
             filename (str): A relative path to a file which is intended to be signed.
+            env_vars (dict): dictionary of environment variables
 
         Raises:
             RuntimeError: If the return code of the script is not equal to 0.
@@ -738,8 +740,14 @@ class SigningService(BaseModel):
         Returns:
             A dictionary as validated by the validate() method.
         """
+        env = {"PULP_SIGNING_KEY_FINGERPRINT": self.pubkey_fingerprint}
+        if env_vars:
+            env.update(env_vars)
         completed_process = subprocess.run(
-            [self.script, filename], env={}, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            [self.script, filename],
+            env=env,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
         )
 
         if completed_process.returncode != 0:
@@ -752,6 +760,30 @@ class SigningService(BaseModel):
 
         return return_value
 
+    async def asign(self, filename, env_vars=None):
+        """Async version of sign."""
+        env = {"PULP_SIGNING_KEY_FINGERPRINT": self.pubkey_fingerprint}
+        if env_vars:
+            env.update(env_vars)
+        process = await asyncio.create_subprocess_exec(
+            self.script,
+            filename,
+            env=env,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.PIPE,
+        )
+
+        stdout, stderr = await process.communicate()
+        if process.returncode != 0:
+            raise RuntimeError(str(stderr))
+
+        try:
+            return_value = json.loads(stdout)
+        except json.JSONDecodeError:
+            raise RuntimeError("The signing service script did not return valid JSON!")
+
+        return return_value
+
     def validate(self):
         """
         Ensure that the external signing script produces the desired behaviour.
