@@ -1,3 +1,9 @@
+import hashlib
+import os
+import re
+from url_normalize import url_normalize
+from urllib.parse import urlparse
+
 from django.db import IntegrityError, models, transaction
 from django_lifecycle import hook, AFTER_UPDATE, BEFORE_DELETE
 
@@ -9,7 +15,7 @@ from pulpcore.app.files import PulpTemporaryUploadedFile
 from pulpcore.cache import Cache
 from dynaconf import settings
 from rest_framework.exceptions import APIException
-from pulpcore.app.models import AutoAddObjPermsMixin, AutoDeleteObjPermsMixin
+from pulpcore.app.models import AutoAddObjPermsMixin
 
 
 class PublicationQuerySet(models.QuerySet):
@@ -334,12 +340,11 @@ class ContentGuard(MasterModel):
                 Cache().delete(base_key=base_paths)
 
 
-class RBACContentGuard(ContentGuard, AutoAddObjPermsMixin, AutoDeleteObjPermsMixin):
+class RBACContentGuard(ContentGuard, AutoAddObjPermsMixin):
     """
     A content guard that protects content based on RBAC permissions.
     """
 
-    ACCESS_POLICY_VIEWSET_NAME = "contentguards/core/rbac"
     TYPE = "rbac"
 
     def permit(self, request):
@@ -358,24 +363,66 @@ class RBACContentGuard(ContentGuard, AutoAddObjPermsMixin, AutoDeleteObjPermsMix
         except APIException as e:
             raise PermissionError(e)
 
-    def add_can_download(self, users, groups):
-        """
-        Adds the can_download permission to users & groups upon content guard creation
-        """
-        if users:
-            self.add_for_users("core.download_rbaccontentguard", users)
-        if groups:
-            self.add_for_groups("core.download_rbaccontentguard", groups)
+    class Meta:
+        default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = (
+            ("download_rbaccontentguard", "Can Download Content"),
+            ("manage_roles_rbaccontentguard", "Can manage role assignments on RBAC content guard"),
+        )
 
-    def remove_can_download(self, users, groups):
-        if users:
-            self.remove_for_users("core.download_rbaccontentguard", users)
-        if groups:
-            self.remove_for_groups("core.download_rbaccontentguard", groups)
+
+def _gen_secret():
+    return os.urandom(32)
+
+
+class ContentRedirectContentGuard(ContentGuard):
+    """
+    Content guard to allow preauthenticated redirects to the content app.
+    """
+
+    TYPE = "content_redirect"
+
+    shared_secret = models.BinaryField(max_length=32, default=_gen_secret)
+
+    def permit(self, request):
+        """
+        Permit preauthenticated redirects from pulp-api.
+        """
+        try:
+            signed_url = request.url
+            validate_token = request.query["validate_token"]
+            hex_salt, hex_digest = validate_token.split(":", 1)
+            salt = bytes.fromhex(hex_salt)
+            digest = bytes.fromhex(hex_digest)
+            url = re.sub(r"\?validate_token=.*$", "", str(signed_url))
+            if not digest == self._get_digest(salt, url):
+                raise PermissionError("Access not authenticated")
+        except (KeyError, ValueError):
+            raise PermissionError("Access not authenticated")
+
+    def preauthenticate_url(self, url, salt=None):
+        """
+        Add validate_token to urls query string.
+        """
+        if not salt:
+            salt = _gen_secret()
+        hex_salt = salt.hex()
+        digest = self._get_digest(salt, url).hex()
+        url = url + f"?validate_token={hex_salt}:{digest}"
+        return url
+
+    def _get_digest(self, salt, url):
+        url_parts = urlparse(url_normalize(url))
+        hasher = hashlib.sha256()
+        hasher.update(salt)
+        hasher.update(url_parts.path.encode())
+        hasher.update(b"?")
+        hasher.update(url_parts.query.encode())
+        hasher.update(self.shared_secret)
+        return hasher.digest()
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
-        permissions = (("download_rbaccontentguard", "Can Download Content"),)
 
 
 class BaseDistribution(MasterModel):
