@@ -142,6 +142,25 @@ class Repository(MasterModel):
 
             return version
 
+    def initialize_new_version(self, new_version):
+        """
+        Initialize the new RepositoryVersion with plugin-provided code.
+
+        This method should be overridden by plugin writers for an opportunity for plugin input. This
+        method is intended to be called with the incomplete
+        :class:`pulpcore.app.models.RepositoryVersion` to validate or modify the content.
+
+        This method does not adjust the value of complete, or save the `RepositoryVersion` itself.
+        Its intent is to allow the plugin writer an opportunity for plugin input before any other
+        actions performed on the new `RepositoryVersion`.
+
+        Args:
+            new_version (pulpcore.app.models.RepositoryVersion): The incomplete RepositoryVersion to
+                finalize.
+
+        """
+        pass
+
     def finalize_new_version(self, new_version):
         """
         Finalize the incomplete RepositoryVersion with plugin-provided code.
@@ -230,10 +249,17 @@ class Repository(MasterModel):
                 version.delete()
 
     @hook(BEFORE_DELETE)
-    def invalidate_cache(self):
+    def invalidate_cache(self, everything=False):
         """Invalidates the cache if repository is present."""
         if settings.CACHE_ENABLED:
             distributions = self.distributions.all()
+            if everything:
+                from .publication import Distribution, Publication
+
+                versions = self.versions.all()
+                pubs = Publication.objects.filter(repository_version__in=versions, complete=True)
+                distributions |= Distribution.objects.filter(publication__in=pubs)
+                distributions |= Distribution.objects.filter(repository_version__in=versions)
             if distributions.exists():
                 base_paths = distributions.values_list("base_path", flat=True)
                 if base_paths:
@@ -280,7 +306,7 @@ class Remote(MasterModel):
         sock_connect_timeout (models.FloatField): Value for aiohttp.ClientTimeout.sock_connect
         sock_read_timeout (models.FloatField): Value for aiohttp.ClientTimeout.sock_read
         headers (models.JSONField): Headers set on the aiohttp.ClientSession
-        rate_limit (models.IntegerField): Limits total download rate in requests per second.
+        rate_limit (models.IntegerField): Limits requests per second for each concurrent downloader
     """
 
     TYPE = "remote"
@@ -386,7 +412,7 @@ class Remote(MasterModel):
                 self._download_throttler = Throttler(rate_limit=self.rate_limit)
                 return self._download_throttler
 
-    def get_downloader(self, remote_artifact=None, url=None, **kwargs):
+    def get_downloader(self, remote_artifact=None, url=None, download_factory=None, **kwargs):
         """
         Get a downloader from either a RemoteArtifact or URL that is configured with this Remote.
 
@@ -400,6 +426,8 @@ class Remote(MasterModel):
             remote_artifact (:class:`~pulpcore.app.models.RemoteArtifact`): The RemoteArtifact to
                 download.
             url (str): The URL to download.
+            download_factory (:class:`~pulpcore.plugin.download.DownloadFactory`): The download
+                factory to be used.
             kwargs (dict): This accepts the parameters of
                 :class:`~pulpcore.plugin.download.BaseDownloader`.
 
@@ -425,18 +453,21 @@ class Remote(MasterModel):
                 kwargs["expected_digests"] = expected_digests
             if remote_artifact.size:
                 kwargs["expected_size"] = remote_artifact.size
-        return self.download_factory.build(url, **kwargs)
+        if download_factory is None:
+            download_factory = self.download_factory
+        return download_factory.build(url, **kwargs)
 
-    def get_remote_artifact_url(self, relative_path=None):
+    def get_remote_artifact_url(self, relative_path=None, request=None):
         """
-        Get the full URL for a RemoteArtifact from a relative path.
+        Get the full URL for a RemoteArtifact from relative path and request.
 
-        This method returns the URL for a RemoteArtifact by concatinating the Remote's url and the
-        relative path.located in the Remote. Plugin writers are expected to override this method
-        when a more complex algorithm is needed to determine the full URL.
+        This method returns the URL for a RemoteArtifact by concatenating the Remote's url and the
+        relative path. Plugin writers are expected to override this method when a more complex
+        algorithm is needed to determine the full URL.
 
         Args:
             relative_path (str): The relative path of a RemoteArtifact
+            request (aiohttp.web.Request): The request object for this relative path
 
         Raises:
             ValueError: If relative_path starts with a '/'.
@@ -970,9 +1001,9 @@ class RepositoryVersion(BaseModel):
         objects and makes new ones with each call.
         """
         with transaction.atomic():
+            RepositoryVersionContentDetails.objects.filter(repository_version=self).delete()
             counts_list = []
             for value, name in RepositoryVersionContentDetails.COUNT_TYPE_CHOICES:
-                RepositoryVersionContentDetails.objects.filter(repository_version=self).delete()
                 if value == RepositoryVersionContentDetails.ADDED:
                     qs = self.added()
                 elif value == RepositoryVersionContentDetails.PRESENT:
@@ -1001,6 +1032,8 @@ class RepositoryVersion(BaseModel):
             raise RuntimeError(
                 _("This Repository version is complete. It cannot be modified further.")
             )
+        repository = self.repository.cast()
+        repository.initialize_new_version(self)
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
@@ -1066,18 +1099,23 @@ class RepositoryVersionContentDetails(models.Model):
     @property
     def content_href(self):
         """
-        Generate URLs for the content types present in the RepositoryVersion.
+        Generate URLs for the content types added, removed, or present in the RepositoryVersion.
 
-        For each content type present in the RepositoryVersion, create the URL of the viewset of
-        that variety of content along with a query parameter which filters it by presence in this
-        RepositoryVersion.
+        For each content type present in or removed from this RepositoryVersion, create the URL of
+        the viewset of that variety of content along with a query parameter which filters it by
+        presence in this RepositoryVersion summary.
 
         Args:
             obj (pulpcore.app.models.RepositoryVersion): The RepositoryVersion being serialized.
         Returns:
             dict: {<pulp_type>: <url>}
         """
-        ctype_model = Content.objects.filter(pulp_type=self.content_type).first().cast().__class__
+        repository = self.repository_version.repository.cast()
+        repository_content = RepositoryContent.objects.filter(repository=repository)
+        ctype_query = Content.objects.filter(
+            pulp_type=self.content_type, pk__in=repository_content.values_list("content", flat=True)
+        )
+        ctype_model = ctype_query.first().cast().__class__
         ctype_view = get_view_name_for_model(ctype_model, "list")
         try:
             ctype_url = reverse(ctype_view)
@@ -1085,7 +1123,7 @@ class RepositoryVersionContentDetails(models.Model):
             # We've hit a content type for which there is no viewset.
             # There's nothing we can do here, except to skip it.
             return
-        repository = self.repository_version.repository.cast()
+
         repository_view = get_view_name_for_model(repository.__class__, "list")
 
         repository_url = reverse(repository_view)
