@@ -7,10 +7,11 @@ import os
 from datetime import timedelta
 from gettext import gettext as _
 
+from django.conf import settings
 from django.contrib.postgres.fields import ArrayField
+from django.core.serializers.json import DjangoJSONEncoder
 from django.db import connection, models
 from django.utils import timezone
-from django.conf import settings
 
 from pulpcore.app.models import (
     AutoAddObjPermsMixin,
@@ -22,6 +23,7 @@ from pulpcore.constants import TASK_CHOICES, TASK_FINAL_STATES, TASK_STATES
 from pulpcore.exceptions import AdvisoryLockError, exception_to_dict
 from pulpcore.tasking.constants import TASKING_CONSTANTS
 
+
 _logger = logging.getLogger(__name__)
 
 
@@ -42,20 +44,23 @@ class WorkerManager(models.Manager):
 
         return self.filter(last_heartbeat__gte=age_threshold)
 
-    def missing_workers(self):
+    def missing_workers(self, age=timedelta(seconds=settings.WORKER_TTL)):
         """
         Returns a queryset of workers meeting the criteria to be considered 'missing'
 
-        To be considered missing, a worker must have a stale timestamp.  Stale is defined here as
-        "beyond the pulp process timeout interval".
+        To be considered missing, a worker must have a stale timestamp.  By default, stale is
+        defined here as longer than the ``settings.WORKER_TTL``, or you can specify age as a
+        timedelta.
+
+        Args:
+            age (datetime.timedelta): Workers who have heartbeats older than this time interval are
+                considered missing.
 
         Returns:
             :class:`django.db.models.query.QuerySet`:  A query set of the Worker objects which
                 are considered by Pulp to be 'missing'.
         """
-        now = timezone.now()
-        age_threshold = now - timedelta(seconds=settings.WORKER_TTL)
-
+        age_threshold = timezone.now() - age
         return self.filter(last_heartbeat__lt=age_threshold)
 
     def resource_managers(self):
@@ -86,6 +91,16 @@ class Worker(BaseModel):
     name = models.TextField(db_index=True, unique=True)
     last_heartbeat = models.DateTimeField(auto_now=True)
 
+    @property
+    def current_task(self):
+        """
+        The task this worker is currently executing, if any.
+
+        Returns:
+            Task: The currently executing task
+        """
+        return self.tasks.filter(state="running").first()
+
     @property
     def online(self):
         """
@@ -169,8 +184,8 @@ class Task(BaseModel, AutoDeleteObjPermsMixin, AutoAddObjPermsMixin):
 
     error = models.JSONField(null=True)
 
-    args = models.JSONField(null=True)
-    kwargs = models.JSONField(null=True)
+    args = models.JSONField(null=True, encoder=DjangoJSONEncoder)
+    kwargs = models.JSONField(null=True, encoder=DjangoJSONEncoder)
 
     worker = models.ForeignKey("Worker", null=True, related_name="tasks", on_delete=models.SET_NULL)
 
@@ -182,8 +197,6 @@ class Task(BaseModel, AutoDeleteObjPermsMixin, AutoAddObjPermsMixin):
     )
     reserved_resources_record = ArrayField(models.CharField(max_length=256), null=True)
 
-    ACCESS_POLICY_VIEWSET_NAME = "tasks"
-
     def __str__(self):
         return "Task: {name} [{state}]".format(name=self.name, state=self.state)
 
@@ -276,6 +289,9 @@ class Task(BaseModel, AutoDeleteObjPermsMixin, AutoAddObjPermsMixin):
 
     class Meta:
         indexes = [models.Index(fields=["pulp_created"])]
+        permissions = [
+            ("manage_roles_task", "Can manage role assignments on task"),
+        ]
 
 
 class TaskGroup(BaseModel):
@@ -317,3 +333,16 @@ class CreatedResource(GenericRelationModel):
     task = models.ForeignKey(
         Task, related_name="created_resources", default=Task.current, on_delete=models.CASCADE
     )
+
+
+class TaskSchedule(BaseModel):
+    name = models.CharField(max_length=256, unique=True, null=False)
+    next_dispatch = models.DateTimeField(default=timezone.now, null=True)
+    dispatch_interval = models.DurationField(null=True)
+    task_name = models.CharField(max_length=256)
+    last_task = models.ForeignKey(Task, null=True, on_delete=models.SET_NULL)
+
+    class Meta:
+        permissions = [
+            ("manage_roles_taskschedule", "Can manage role assignments on task schedules"),
+        ]
