@@ -2,7 +2,6 @@ from gettext import gettext as _
 
 from django.db.models import Q
 from drf_spectacular.utils import extend_schema_field
-from guardian.shortcuts import get_users_with_perms, get_groups_with_perms
 from rest_framework import serializers
 from rest_framework.validators import UniqueValidator
 
@@ -11,12 +10,14 @@ from pulpcore.app.serializers import (
     BaseURLField,
     DetailIdentityField,
     DetailRelatedField,
+    GetOrCreateSerializerMixin,
     LabelsField,
     ModelSerializer,
     RepositoryVersionRelatedField,
     validate_unknown_fields,
 )
 from pulpcore.app.serializers.user import GroupUserSerializer, GroupSerializer
+from pulpcore.app.role_util import get_users_with_perms, get_groups_with_perms
 
 
 class PublicationSerializer(ModelSerializer):
@@ -112,11 +113,20 @@ class RBACContentGuardPermissionSerializer(serializers.Serializer):
     groupnames = serializers.ListField(default=[])
 
 
+class ContentRedirectContentGuardSerializer(ContentGuardSerializer, GetOrCreateSerializerMixin):
+    """
+    A serializer for ContentRedirectContentGuard.
+    """
+
+    class Meta(ContentGuardSerializer.Meta):
+        model = models.ContentRedirectContentGuard
+
+
 class DistributionSerializer(ModelSerializer):
     """
     The Serializer for the Distribution model.
 
-    The serializer deliberately omits the the `publication` and `repository_version` field due to
+    The serializer deliberately omits the `publication` and `repository_version` field due to
     plugins typically requiring one or the other but not both.
 
     To include the ``publication`` field, it is recommended plugins define the field::
@@ -226,14 +236,14 @@ class DistributionSerializer(ModelSerializer):
     def validate(self, data):
         super().validate(data)
 
-        repository_provided = data.get("repository", None) or (
-            self.instance and self.instance.repository
+        repository_provided = data.get(
+            "repository", (self.instance and self.instance.repository) or None
         )
-        repository_version_provided = data.get("repository_version", None) or (
-            self.instance and self.instance.repository_version
+        repository_version_provided = data.get(
+            "repository_version", (self.instance and self.instance.repository_version) or None
         )
-        publication_provided = data.get("publication", None) or (
-            self.instance and self.instance.publication
+        publication_provided = data.get(
+            "publication", (self.instance and self.instance.publication) or None
         )
 
         if publication_provided and repository_version_provided:
@@ -250,14 +260,12 @@ class DistributionSerializer(ModelSerializer):
                     "may be used simultaneously."
                 )
             )
-        # TODO: https://pulp.plan.io/issues/8762
-
-        # elif repository_provided and publication_provided:
-        #     raise serializers.ValidationError(
-        #         _(
-        #             "Only one of the attributes 'repository' and 'publication' "
-        #             "may be used simultaneously."
-        #         )
-        #     )
 
+        elif repository_provided and publication_provided:
+            raise serializers.ValidationError(
+                _(
+                    "Only one of the attributes 'repository' and 'publication' "
+                    "may be used simultaneously."
+                )
+            )
         return data
