@@ -1,5 +1,6 @@
-from gettext import gettext as _
 import os
+from gettext import gettext as _
+from urllib.parse import urlparse
 
 from rest_framework import fields, serializers
 from rest_framework.validators import UniqueValidator
@@ -201,7 +202,7 @@ class RemoteSerializer(ModelSerializer):
         required=False,
     )
     rate_limit = serializers.IntegerField(
-        help_text=_("Limits total download rate in requests per second"),
+        help_text=_("Limits requests per second for each concurrent downloader"),
         allow_null=True,
         required=False,
     )
@@ -222,6 +223,15 @@ class RemoteSerializer(ModelSerializer):
         Returns:
             The validated value.
         """
+        parsed_url = urlparse(url)
+        if parsed_url.username or parsed_url.password:
+            raise serializers.ValidationError(
+                _(
+                    "The remote url contains username or password. Please use remote username or "
+                    "password instead."
+                )
+            )
+
         if not url.lower().startswith("file://"):
             return url
 
