@@ -60,6 +60,9 @@ SECRET_KEY = True
 # Key used to encrypt fields in the database
 DB_ENCRYPTION_KEY = "/etc/pulp/certs/database_fields.symmetric.key"
 
+# API Root
+API_ROOT = "/pulp/"
+
 # Application definition
 
 INSTALLED_APPS = [
@@ -118,6 +121,7 @@ MIDDLEWARE = [
 AUTHENTICATION_BACKENDS = [
     "django.contrib.auth.backends.ModelBackend",
     "guardian.backends.ObjectPermissionBackend",
+    "pulpcore.backends.ObjectRolePermissionBackend",
 ]
 
 # Disable django guardian anonymous user
@@ -199,8 +203,8 @@ DATABASES = {
 
 # Redis default config
 REDIS_URL = None
-REDIS_HOST = "localhost"
-REDIS_PORT = 6379
+REDIS_HOST = None
+REDIS_PORT = None
 REDIS_DB = 0
 REDIS_PASSWORD = None
 REDIS_SSL = False
@@ -253,12 +257,10 @@ ALLOWED_IMPORT_PATHS = []
 
 ALLOWED_EXPORT_PATHS = []
 
-ALLOW_SHARED_TASK_RESOURCES = True
-
 PROFILE_STAGES_API = False
 
 # https://docs.pulpproject.org/pulpcore/configuration/settings.html#pulp-cache
-CACHE_ENABLED = True
+CACHE_ENABLED = False
 CACHE_SETTINGS = {
     "EXPIRES_TTL": 600,  # 10 minutes
 }
@@ -312,6 +314,19 @@ settings = dynaconf.DjangoDynaconf(
 
 _logger = getLogger(__name__)
 
+# Post-dyna-conf check if user provided a redis connection when enabling cache
+try:
+    if CACHE_ENABLED:
+        REDIS_URL or (REDIS_HOST and REDIS_PORT)
+except NameError:
+    raise ImproperlyConfigured(
+        _(
+            "CACHE_ENABLED is enabled but it requires to have REDIS configured. Please check "
+            "https://docs.pulpproject.org/pulpcore/configuration/settings.html#redis-settings "
+            "for more information."
+        )
+    )
+
 try:
     CONTENT_ORIGIN
 except NameError:
@@ -324,7 +339,8 @@ except NameError:
     )
 
 if not (
-    Path(sys.argv[0]).name == "sphinx-build"
+    Path(sys.argv[0]).name == "pytest"
+    or Path(sys.argv[0]).name == "sphinx-build"
     or (len(sys.argv) >= 2 and sys.argv[1] == "collectstatic")
 ):
     try:
@@ -418,3 +434,17 @@ if not (len(sys.argv) >= 2 and sys.argv[1] in _SKIPPED_COMMANDS_FOR_CONTENT_CHEC
         pass
     finally:
         connection.close()
+
+
+if not API_ROOT.startswith("/"):
+    i8ln_msg = _("The API_ROOT must start with a '/', currently it is '{API_ROOT}'")
+    raise ImproperlyConfigured(i8ln_msg.format(API_ROOT=API_ROOT))
+
+if not API_ROOT.endswith("/"):
+    i8ln_msg = _("The API_ROOT must end with a '/', currently it is '{API_ROOT}'")
+    raise ImproperlyConfigured(i8ln_msg.format(API_ROOT=API_ROOT))
+
+settings.set("V3_API_ROOT", settings.API_ROOT + "api/v3/")  # Not user configurable
+settings.set(
+    "V3_API_ROOT_NO_FRONT_SLASH", settings.V3_API_ROOT.lstrip("/")
+)  # Not user configurable
