@@ -1,7 +1,6 @@
 from drf_spectacular.utils import extend_schema
 from rest_framework.views import APIView
 
-from pulpcore.app.loggers import deprecation_logger
 from pulpcore.app.response import OperationPostponedResponse
 from pulpcore.app.serializers import AsyncOperationResponseSerializer
 from pulpcore.app.tasks import orphan_cleanup
@@ -20,11 +19,6 @@ class OrphansView(APIView):
         """
         Cleans up all the Content and Artifact orphans in the system
         """
-        deprecation_logger.warning(
-            "The `DELETE /pulp/api/v3/orphans/` call is deprecated. Use"
-            "`POST /pulp/api/v3/orphans/cleanup/` instead."
-        )
-
         task = dispatch(orphan_cleanup)
 
         return OperationPostponedResponse(task, request)
