@@ -4,6 +4,7 @@ from .base import (  # noqa
     AsyncUpdateMixin,
     BaseFilterSet,
     NamedModelViewSet,
+    RolesMixin,
     NAME_FILTER_OPTIONS,
 )
 
@@ -50,6 +51,7 @@ from .publication import (  # noqa
     PublicationFilter,
     PublicationViewSet,
     RBACContentGuardViewSet,
+    ContentRedirectContentGuardViewSet,
 )
 from .reclaim import ReclaimSpaceViewSet  # noqa
 from .repository import (  # noqa
@@ -62,12 +64,15 @@ from .repository import (  # noqa
     RepositoryVersionViewSet,
     ListRepositoryVersionViewSet,
 )
-from .task import TaskViewSet, TaskGroupViewSet, WorkerViewSet  # noqa
+from .task import TaskViewSet, TaskGroupViewSet, TaskScheduleViewSet, WorkerViewSet  # noqa
 from .upload import UploadViewSet  # noqa
 from .user import (  # noqa
     GroupViewSet,
+    GroupRoleViewSet,
     GroupUserViewSet,
     GroupModelPermissionViewSet,
     GroupObjectPermissionViewSet,
+    RoleViewSet,
     UserViewSet,
+    UserRoleViewSet,
 )
