@@ -2,22 +2,26 @@ import warnings
 from gettext import gettext as _
 from urllib.parse import urlparse
 
+from django.db import transaction
 from django.core.exceptions import FieldDoesNotExist, FieldError, ValidationError
 from django.forms.utils import ErrorList
 from django.urls import Resolver404, resolve
+from django.contrib.contenttypes.models import ContentType
 from django_filters.rest_framework import DjangoFilterBackend, filterset
-from drf_spectacular.utils import extend_schema
-from guardian.shortcuts import get_objects_for_user
+from drf_spectacular.utils import extend_schema, inline_serializer
 from rest_framework import viewsets
+from rest_framework.decorators import action
 from rest_framework.filters import OrderingFilter
 from rest_framework.generics import get_object_or_404
+from rest_framework.response import Response
 from pulpcore.openapi import PulpAutoSchema
-from rest_framework.serializers import ValidationError as DRFValidationError
+from rest_framework.serializers import ValidationError as DRFValidationError, ListField, CharField
 
 from pulpcore.app import tasks
 from pulpcore.app.models import MasterModel
+from pulpcore.app.models.role import GroupRole, UserRole
 from pulpcore.app.response import OperationPostponedResponse
-from pulpcore.app.serializers import AsyncOperationResponseSerializer
+from pulpcore.app.serializers import AsyncOperationResponseSerializer, NestedRoleSerializer
 from pulpcore.tasking.tasks import dispatch
 
 # These should be used to prevent duplication and keep things consistent
@@ -69,7 +73,7 @@ class StableOrderingFilter(OrderingFilter):
         The `ordering` query parameter can be overridden by setting the `ordering_param` value on
         the OrderingFilter or by specifying an `ORDERING_PARAM` value in the API settings.
         """
-        ordering = super(StableOrderingFilter, self).get_ordering(request, queryset, view)
+        ordering = super().get_ordering(request, queryset, view)
         try:
             field = queryset.model._meta.get_field("pulp_created")
         except FieldDoesNotExist:
@@ -80,6 +84,19 @@ class StableOrderingFilter(OrderingFilter):
 
         return list(ordering) + ["-" + field.name]
 
+    def get_default_valid_fields(self, queryset, view, context={}):
+        """
+        Ensure that default valid fields don't include fields not on the queryset model.
+
+        When `ordering_fields` is not specified this method is called to find the fields to order
+        by. It uses the default serializer class's fields to find the `ordering_fields` which could
+        include fields from related objects not present in the queryset.
+        """
+        # Returns a list of tuples (field_name, field_label)
+        valid_fields = super().get_default_valid_fields(queryset, view, context)
+        model_fields = {field.name for field in queryset.model._meta.get_fields()}
+        return [field for field in valid_fields if field[0] in model_fields]
+
 
 class NamedModelViewSet(viewsets.GenericViewSet):
     """
@@ -331,9 +348,10 @@ class NamedModelViewSet(viewsets.GenericViewSet):
                 filters[lookup] = self.kwargs[key]
             qs = qs.filter(**filters)
 
-        permission_name = getattr(self, "queryset_filtering_required_permission", None)
-        if permission_name:
-            qs = get_objects_for_user(self.request.user, permission_name, klass=qs)
+        if getattr(self, "request", None):
+            for permission_class in self.get_permissions():
+                if hasattr(permission_class, "scope_queryset"):
+                    qs = permission_class.scope_queryset(self, qs)
 
         return qs
 
@@ -494,12 +512,122 @@ class AsyncRemoveMixin(AsyncReservedObjectMixin):
         return OperationPostponedResponse(task, request)
 
 
+class RolesMixin:
+    @extend_schema(
+        description="List roles assigned to this object.",
+        responses={
+            200: inline_serializer(
+                name="ObjectRolesSerializer",
+                fields={"roles": ListField(child=NestedRoleSerializer())},
+            )
+        },
+    )
+    @action(detail=True, methods=["get"])
+    def list_roles(self, request, pk):
+        obj = self.get_object()
+        obj_type = ContentType.objects.get_for_model(obj)
+        user_qs = UserRole.objects.filter(
+            content_type_id=obj_type.id, object_id=obj.pk
+        ).select_related("user", "role")
+        group_qs = GroupRole.objects.filter(
+            content_type_id=obj_type.id, object_id=obj.pk
+        ).select_related("group", "role")
+        roles = {}
+        for user_role in user_qs:
+            if user_role.role.name not in roles:
+                roles[user_role.role.name] = {
+                    "role": user_role.role.name,
+                    "users": [],
+                    "groups": [],
+                }
+            roles[user_role.role.name]["users"].append(user_role.user.username)
+        for group_role in group_qs:
+            if group_role.role.name not in roles:
+                roles[group_role.role.name] = {
+                    "role": group_role.role.name,
+                    "users": [],
+                    "groups": [],
+                }
+            roles[group_role.role.name]["groups"].append(group_role.group.name)
+        result = {"roles": list(roles.values())}
+        return Response(result)
+
+    @extend_schema(
+        description="Add a role for this object to users/groups.",
+        responses={201: NestedRoleSerializer},
+    )
+    @action(detail=True, methods=["post"], serializer_class=NestedRoleSerializer)
+    def add_role(self, request, pk):
+        obj = self.get_object()
+        serializer = NestedRoleSerializer(
+            data=request.data, context={"request": request, "content_object": obj, "assign": True}
+        )
+        serializer.is_valid(raise_exception=True)
+        with transaction.atomic():
+            if serializer.validated_data["users"]:
+                UserRole.objects.bulk_create(
+                    [
+                        UserRole(
+                            content_object=obj,
+                            user=user,
+                            role=serializer.validated_data["role"],
+                        )
+                        for user in serializer.validated_data["users"]
+                    ]
+                )
+            if serializer.validated_data["groups"]:
+                GroupRole.objects.bulk_create(
+                    [
+                        GroupRole(
+                            content_object=obj,
+                            group=group,
+                            role=serializer.validated_data["role"],
+                        )
+                        for group in serializer.validated_data["groups"]
+                    ]
+                )
+        return Response(serializer.data, status=201)
+
+    @extend_schema(
+        description="Remove a role for this object from users/groups.",
+        responses={201: NestedRoleSerializer},
+    )
+    @action(detail=True, methods=["post"], serializer_class=NestedRoleSerializer)
+    def remove_role(self, request, pk):
+        obj = self.get_object()
+        serializer = NestedRoleSerializer(
+            data=request.data, context={"request": request, "content_object": obj, "assign": False}
+        )
+        serializer.is_valid(raise_exception=True)
+        with transaction.atomic():
+            UserRole.objects.filter(pk__in=serializer.user_role_pks).delete()
+            GroupRole.objects.filter(pk__in=serializer.group_role_pks).delete()
+        return Response(serializer.data, status=201)
+
+    @extend_schema(
+        description="List permissions available to the current user on this object.",
+        responses={
+            200: inline_serializer(
+                name="MyPermissionsSerializer", fields={"permissions": ListField(child=CharField())}
+            )
+        },
+    )
+    @action(detail=True, methods=["get"])
+    def my_permissions(self, request, pk=None):
+        obj = self.get_object()
+        app_label = obj._meta.app_label
+        permissions = [
+            ".".join((app_label, codename)) for codename in request.user.get_all_permissions(obj)
+        ]
+        return Response({"permissions": permissions})
+
+
 class BaseFilterSet(filterset.FilterSet):
     """
     Class to override django_filter's FilterSet and provide a way to set help text
 
     By default, this class will use predefined text and the field name to create help text for the
-    filter. However, this can be overriden by setting a help_text dict with the the field name
+    filter. However, this can be overriden by setting a help_text dict with the field name
     mapped to some help text:
 
         help_text = {'name__in': 'Lorem ipsum dolor', 'pulp_last_updated__lt': 'blah blah'}
