@@ -3,19 +3,25 @@ from gettext import gettext as _
 from django_filters.rest_framework import DjangoFilterBackend, filters
 from drf_spectacular.utils import extend_schema
 from rest_framework import mixins, status
+from rest_framework.decorators import action
 from rest_framework.filters import OrderingFilter
 from rest_framework.response import Response
 from rest_framework.serializers import ValidationError
 
-from pulpcore.app.models import Task, TaskGroup, Worker
+from pulpcore.app.models import Task, TaskGroup, TaskSchedule, Worker
+from pulpcore.app.response import OperationPostponedResponse
 from pulpcore.app.serializers import (
+    AsyncOperationResponseSerializer,
     MinimalTaskSerializer,
+    PurgeSerializer,
     TaskCancelSerializer,
+    TaskGroupSerializer,
+    TaskScheduleSerializer,
     TaskSerializer,
     WorkerSerializer,
-    TaskGroupSerializer,
 )
-from pulpcore.app.viewsets import BaseFilterSet, NamedModelViewSet
+from pulpcore.app.tasks import purge
+from pulpcore.app.viewsets import BaseFilterSet, NamedModelViewSet, RolesMixin
 from pulpcore.app.viewsets.base import DATETIME_FILTER_OPTIONS, NAME_FILTER_OPTIONS
 from pulpcore.app.viewsets.custom_filters import (
     HyperlinkRelatedFilter,
@@ -24,6 +30,7 @@ from pulpcore.app.viewsets.custom_filters import (
     CreatedResourcesFilter,
 )
 from pulpcore.constants import TASK_INCOMPLETE_STATES, TASK_STATES, TASK_CHOICES
+from pulpcore.tasking.tasks import dispatch
 from pulpcore.tasking.util import cancel as cancel_task
 
 
@@ -58,7 +65,11 @@ class TaskFilter(BaseFilterSet):
 
 
 class TaskViewSet(
-    NamedModelViewSet, mixins.RetrieveModelMixin, mixins.ListModelMixin, mixins.DestroyModelMixin
+    NamedModelViewSet,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    mixins.DestroyModelMixin,
+    RolesMixin,
 ):
     queryset = Task.objects.all()
     endpoint_name = "tasks"
@@ -73,7 +84,7 @@ class TaskViewSet(
         "statements": [
             {"action": ["list"], "principal": "authenticated", "effect": "allow"},
             {
-                "action": ["retrieve"],
+                "action": ["retrieve", "my_permissions"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": "has_model_or_obj_perms:core.view_task",
@@ -90,15 +101,39 @@ class TaskViewSet(
                 "effect": "allow",
                 "condition": "has_model_or_obj_perms:core.change_task",
             },
-        ],
-        "permissions_assignment": [
+            # 'purge' is filtered by current-user and core.delete_task permissions at the queryset
+            # level, and needs no extra protections here
             {
-                "function": "add_for_object_creator",
-                "parameters": None,
-                "permissions": ["core.view_task", "core.change_task", "core.delete_task"],
+                "action": ["purge"],
+                "principal": "authenticated",
+                "effect": "allow",
+            },
+            {
+                "action": ["list_roles", "add_role", "remove_role"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": "has_model_or_obj_perms:core.manage_roles_task",
+            },
+        ],
+        "creation_hooks": [
+            {
+                "function": "add_roles_for_object_creator",
+                "parameters": {"roles": "core.task_owner"},
             }
         ],
     }
+    LOCKED_ROLES = {
+        "core.task_owner": {
+            "description": "Allow all actions on a task.",
+            "permissions": [
+                "core.view_task",
+                "core.change_task",
+                "core.delete_task",
+                "core.manage_roles_task",
+            ],
+        },
+        "core.task_viewer": ["core.view_task"],
+    }
 
     @extend_schema(
         description="This operation cancels a task.",
@@ -133,6 +168,28 @@ class TaskViewSet(
             return TaskCancelSerializer
         return super().get_serializer_class()
 
+    @extend_schema(
+        description=(
+            "Trigger an asynchronous task that deletes completed tasks that finished prior"
+            " to a specified timestamp (tech-preview, may change in the future)."
+        ),
+        summary="Purge Completed Tasks",
+        operation_id="tasks_purge",
+        request=PurgeSerializer,
+        responses={202: AsyncOperationResponseSerializer},
+    )
+    @action(detail=False, methods=["post"])
+    def purge(self, request):
+        """
+        Purge task-records for tasks in 'final' states.
+        """
+        serializer = PurgeSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        task = dispatch(
+            purge, args=[serializer.data["finished_before"], list(serializer.data["states"])]
+        )
+        return OperationPostponedResponse(task, request)
+
 
 class TaskGroupFilter(BaseFilterSet):
     class Meta:
@@ -186,3 +243,70 @@ class WorkerViewSet(NamedModelViewSet, mixins.RetrieveModelMixin, mixins.ListMod
     http_method_names = ["get", "options"]
     lookup_value_regex = "[^/]+"
     filterset_class = WorkerFilter
+
+
+class TaskScheduleFilter(BaseFilterSet):
+    name = filters.CharFilter()
+    task_name = filters.CharFilter()
+
+    class Meta:
+        model = TaskSchedule
+        fields = {
+            "name": ["exact", "contains"],
+            "task_name": ["exact", "contains"],
+        }
+
+
+class TaskScheduleViewSet(
+    NamedModelViewSet,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    RolesMixin,
+):
+    """
+    ViewSet to monitor task schedules.
+
+    NOTE: This feature is in tech-preview and may change in backwards incompatible ways.
+    """
+
+    queryset = TaskSchedule.objects.all()
+    endpoint_name = "task-schedules"
+    filterset_class = TaskScheduleFilter
+    serializer_class = TaskScheduleSerializer
+    filter_backends = (OrderingFilter, DjangoFilterBackend)
+    ordering = "-pulp_created"
+    queryset_filtering_required_permission = "core.view_taskschedule"
+
+    DEFAULT_ACCESS_POLICY = {
+        "statements": [
+            {"action": ["list"], "principal": "authenticated", "effect": "allow"},
+            {
+                "action": ["retrieve", "my_permissions"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": "has_model_or_obj_perms:core.view_taskschedule",
+            },
+            {
+                "action": ["list_roles", "add_role", "remove_role"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": "has_model_or_obj_perms:core.manage_roles_taskschedule",
+            },
+        ],
+        "creation_hooks": [
+            {
+                "function": "add_roles_for_object_creator",
+                "parameters": {"roles": "core.taskschedule_owner"},
+            }
+        ],
+    }
+    LOCKED_ROLES = {
+        "core.taskschedule_owner": {
+            "description": "Allow all actions on a taskschedule.",
+            "permissions": [
+                "core.view_taskschedule",
+                "core.manage_roles_taskschedule",
+            ],
+        },
+        "core.taskschedule_viewer": ["core.view_taskschedule"],
+    }
