@@ -1,25 +1,33 @@
 from gettext import gettext as _
 
 from django.contrib.auth import get_user_model
-from django.contrib.auth.models import Group, Permission
+from django.contrib.auth.models import Permission
+from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import FieldError
 from django.db import IntegrityError
 from django.shortcuts import get_object_or_404
-from django_filters.rest_framework import DjangoFilterBackend
+from django_filters.rest_framework import DjangoFilterBackend, filters
 from drf_spectacular.utils import extend_schema
 from guardian.models.models import GroupObjectPermission
 from rest_framework import mixins, status
 from rest_framework.decorators import action
+from rest_framework.exceptions import PermissionDenied
 from rest_framework.filters import OrderingFilter
+from rest_framework.permissions import SAFE_METHODS
 from rest_framework.response import Response
 from rest_framework.serializers import ValidationError
 
-from pulpcore.app.viewsets import BaseFilterSet, NamedModelViewSet
-from pulpcore.app.serializers.user import (
+from pulpcore.app.models import Group
+from pulpcore.app.models.role import GroupRole, Role, UserRole
+from pulpcore.app.viewsets import BaseFilterSet, NamedModelViewSet, RolesMixin, NAME_FILTER_OPTIONS
+from pulpcore.app.serializers import (
     GroupSerializer,
     GroupUserSerializer,
+    GroupRoleSerializer,
     PermissionSerializer,
+    RoleSerializer,
     UserSerializer,
+    UserRoleSerializer,
 )
 
 User = get_user_model()
@@ -42,7 +50,14 @@ class UserFilter(BaseFilterSet):
         }
 
 
-class UserViewSet(NamedModelViewSet, mixins.RetrieveModelMixin, mixins.ListModelMixin):
+class UserViewSet(
+    NamedModelViewSet,
+    mixins.CreateModelMixin,
+    mixins.UpdateModelMixin,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    mixins.DestroyModelMixin,
+):
     """
     ViewSet for User.
 
@@ -51,6 +66,7 @@ class UserViewSet(NamedModelViewSet, mixins.RetrieveModelMixin, mixins.ListModel
     """
 
     endpoint_name = "users"
+    router_lookup = "user"
     filterset_class = UserFilter
     filter_backends = (OrderingFilter, DjangoFilterBackend)
     serializer_class = UserSerializer
@@ -91,6 +107,7 @@ class GroupViewSet(
     mixins.RetrieveModelMixin,
     mixins.ListModelMixin,
     mixins.DestroyModelMixin,
+    RolesMixin,
 ):
     """
     ViewSet for Group.
@@ -106,7 +123,7 @@ class GroupViewSet(
     serializer_class = GroupSerializer
     queryset = Group.objects.all()
     ordering = ("name",)
-    queryset_filtering_required_permission = "auth.view_group"
+    queryset_filtering_required_permission = "core.view_group"
 
     DEFAULT_ACCESS_POLICY = {
         "statements": [
@@ -119,51 +136,66 @@ class GroupViewSet(
                 "action": ["create"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_model_perms:auth.add_group",
+                "condition": "has_model_perms:core.add_group",
             },
             {
-                "action": ["retrieve"],
+                "action": ["retrieve", "my_permissions"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_model_or_obj_perms:auth.view_group",
+                "condition": "has_model_or_obj_perms:core.view_group",
             },
             {
                 "action": ["update", "partial_update"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_model_or_obj_perms:auth.change_group",
+                "condition": "has_model_or_obj_perms:core.change_group",
             },
             {
                 "action": ["destroy"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_model_or_obj_perms:auth.delete_group",
+                "condition": "has_model_or_obj_perms:core.delete_group",
+            },
+            {
+                "action": ["list_roles", "add_role", "remove_role"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": ["has_model_or_obj_perms:core.manage_roles_group"],
             },
         ],
-        "permissions_assignment": [
+        "creation_hooks": [
             {
-                "function": "add_for_object_creator",
-                "parameters": None,
-                "permissions": [
-                    "auth.view_group",
-                    "auth.change_group",
-                    "auth.delete_group",
-                ],
+                "function": "add_roles_for_object_creator",
+                "parameters": {"roles": "core.group_owner"},
             },
         ],
     }
 
+    LOCKED_ROLES = {
+        "core.group_creator": [
+            "core.add_group",
+        ],
+        "core.group_owner": [
+            "core.view_group",
+            "core.change_group",
+            "core.delete_group",
+            "core.manage_roles_group",
+        ],
+        "core.group_viewer": [
+            "core.view_group",
+        ],
+    }
+
 
 class GroupModelPermissionViewSet(NamedModelViewSet):
     """
-    ViewSet for Model Permissions that belongs to a Group.
+    ViewSet for Model Permissions that belong to a Group.
 
     NOTE: This API endpoint is in "tech preview" and subject to change
 
     """
 
     endpoint_name = "model_permissions"
-    nest_prefix = "groups"
     router_lookup = "model_permission"
     parent_viewset = GroupViewSet
     parent_lookup_kwargs = {"group_pk": "group__pk"}
@@ -247,14 +279,13 @@ class GroupModelPermissionViewSet(NamedModelViewSet):
 
 class GroupObjectPermissionViewSet(NamedModelViewSet):
     """
-    ViewSet for Object Permissions that belongs to a Group.
+    ViewSet for Object Permissions that belong to a Group.
 
     NOTE: This API endpoint is in "tech preview" and subject to change
 
     """
 
     endpoint_name = "object_permissions"
-    nest_prefix = "groups"
     router_lookup = "object_permission"
     parent_viewset = GroupViewSet
     parent_lookup_kwargs = {"group_pk": "group__pk"}
@@ -360,14 +391,13 @@ class GroupObjectPermissionViewSet(NamedModelViewSet):
 
 class GroupUserViewSet(NamedModelViewSet):
     """
-    ViewSet for Users that belongs to a Group.
+    ViewSet for Users that belong to a Group.
 
     NOTE: This API endpoint is in "tech preview" and subject to change
 
     """
 
     endpoint_name = "users"
-    nest_prefix = "groups"
     router_lookup = "user"
     parent_viewset = GroupViewSet
     parent_lookup_kwargs = {"group_pk": "groups__pk"}
@@ -380,22 +410,22 @@ class GroupUserViewSet(NamedModelViewSet):
                 "action": ["list"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_group_model_or_obj_perms:auth.view_group",
+                "condition": "has_group_model_or_obj_perms:core.view_group",
             },
             {
                 "action": ["create"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_group_model_or_obj_perms:auth.change_group",
+                "condition": "has_group_model_or_obj_perms:core.change_group",
             },
             {
                 "action": ["destroy"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_group_model_or_obj_perms:auth.change_group",
+                "condition": "has_group_model_or_obj_perms:core.change_group",
             },
         ],
-        "permissions_assignment": [],
+        "creation_hooks": [],
     }
 
     def list(self, request, group_pk):
@@ -440,3 +470,157 @@ class GroupUserViewSet(NamedModelViewSet):
         group.user_set.remove(user)
         group.save()
         return Response(status=status.HTTP_204_NO_CONTENT)
+
+
+class RoleFilter(BaseFilterSet):
+    """
+    FilterSet for Role.
+    """
+
+    class Meta:
+        model = Role
+        fields = {
+            "name": NAME_FILTER_OPTIONS,
+            "locked": ["exact"],
+        }
+
+
+class RoleViewSet(
+    NamedModelViewSet,
+    mixins.CreateModelMixin,
+    mixins.UpdateModelMixin,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    mixins.DestroyModelMixin,
+):
+    """
+    ViewSet for Role.
+
+    NOTE: This API endpoint is in "tech preview" and subject to change
+
+    """
+
+    endpoint_name = "roles"
+    router_lookup = "role"
+    filterset_class = RoleFilter
+    filter_backends = (OrderingFilter, DjangoFilterBackend)
+    serializer_class = RoleSerializer
+    queryset = Role.objects.all()
+    ordering = ("name",)
+
+    def get_object(self, **kwargs):
+        instance = super().get_object(**kwargs)
+        if instance.locked and self.request.method not in SAFE_METHODS:
+            raise PermissionDenied(detail=_("The role is locked."))
+        return instance
+
+
+class _CharInFilter(filters.BaseInFilter, filters.CharFilter):
+    pass
+
+
+class NestedRoleFilter(BaseFilterSet):
+    """
+    FilterSet for Roles nested under users / groups.
+    """
+
+    # For some reason the generated Filters don't work here
+    role = filters.CharFilter(field_name="role__name")
+    role__in = _CharInFilter(field_name="role__name", lookup_expr="in")
+    role__contains = filters.CharFilter(field_name="role__name", lookup_expr="contains")
+    role__icontains = filters.CharFilter(field_name="role__name", lookup_expr="icontains")
+    role__startswith = filters.CharFilter(field_name="role__name", lookup_expr="startswith")
+    content_object = filters.CharFilter(
+        label="content_object", method="content_object_filter_function"
+    )
+
+    def content_object_filter_function(self, queryset, name, value):
+        if value == "null":
+            return queryset.filter(object_id=None)
+        else:
+            try:
+                obj = NamedModelViewSet.get_resource(value)
+            except ValidationError:
+                raise ValidationError(_("Invalid value for 'content_object': {}.").format(value))
+            obj_type = ContentType.objects.get_for_model(obj, for_concrete_model=False)
+            return queryset.filter(content_type_id=obj_type.id, object_id=obj.pk)
+
+    class Meta:
+        fields = (
+            "role",
+            "role__in",
+            "role__contains",
+            "role__icontains",
+            "role__startswith",
+            "content_object",
+        )
+
+
+class UserRoleFilter(NestedRoleFilter):
+    """
+    FilterSet for UserRole.
+    """
+
+    class Meta:
+        model = UserRole
+        fields = NestedRoleFilter.Meta.fields
+
+
+class UserRoleViewSet(
+    NamedModelViewSet,
+    mixins.CreateModelMixin,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    mixins.DestroyModelMixin,
+):
+    """
+    ViewSet for UserRole.
+
+    NOTE: This API endpoint is in "tech preview" and subject to change
+
+    """
+
+    endpoint_name = "roles"
+    router_lookup = "user"
+    parent_viewset = UserViewSet
+    parent_lookup_kwargs = {"user_pk": "user__pk"}
+    filterset_class = UserRoleFilter
+    filter_backends = (OrderingFilter, DjangoFilterBackend)
+    serializer_class = UserRoleSerializer
+    queryset = UserRole.objects.all()
+    ordering = ("-pulp_created",)
+
+
+class GroupRoleFilter(NestedRoleFilter):
+    """
+    FilterSet for GroupRole.
+    """
+
+    class Meta:
+        model = GroupRole
+        fields = NestedRoleFilter.Meta.fields
+
+
+class GroupRoleViewSet(
+    NamedModelViewSet,
+    mixins.CreateModelMixin,
+    mixins.RetrieveModelMixin,
+    mixins.ListModelMixin,
+    mixins.DestroyModelMixin,
+):
+    """
+    ViewSet for GroupRole.
+
+    NOTE: This API endpoint is in "tech preview" and subject to change
+
+    """
+
+    endpoint_name = "roles"
+    router_lookup = "group"
+    parent_viewset = GroupViewSet
+    parent_lookup_kwargs = {"group_pk": "group__pk"}
+    filterset_class = GroupRoleFilter
+    filter_backends = (OrderingFilter, DjangoFilterBackend)
+    serializer_class = GroupRoleSerializer
+    queryset = GroupRole.objects.all()
+    ordering = ("-pulp_created",)
