@@ -39,7 +39,7 @@ from pulpcore.app.models import (  # noqa: E402: module level not at top of file
 from pulpcore.exceptions import UnsupportedDigestValidationError  # noqa: E402
 
 from jinja2 import Template  # noqa: E402: module level not at top of file
-from pulpcore.cache import ContentCache  # noqa: E402
+from pulpcore.cache import AsyncContentCache  # noqa: E402
 
 log = logging.getLogger(__name__)
 
@@ -95,6 +95,7 @@ class Handler:
     hop_by_hop_headers = [
         "connection",
         "content-encoding",
+        "content-length",
         "keep-alive",
         "public",
         "proxy-authenticate",
@@ -184,7 +185,7 @@ class Handler:
                 if not present:
                     await cached.set(guard_key, str(guard), base_key=base_key)
 
-    @ContentCache(
+    @AsyncContentCache(
         base_key=lambda req, cac: Handler.find_base_path_cached(req, cac),
         auth=lambda req, cac, bk: Handler.auth_cached(req, cac, bk),
     )
@@ -509,7 +510,7 @@ class Handler:
                 pass
             else:
                 if ca.artifact:
-                    return await self._serve_content_artifact(ca, headers)
+                    return await self._serve_content_artifact(ca, headers, request)
                 else:
                     return await self._stream_content_artifact(
                         request, StreamResponse(headers=headers), ca
@@ -537,7 +538,7 @@ class Handler:
                     pass
                 else:
                     if ca.artifact:
-                        return await self._serve_content_artifact(ca, headers)
+                        return await self._serve_content_artifact(ca, headers, request)
                     else:
                         return await self._stream_content_artifact(
                             request, StreamResponse(headers=headers), ca
@@ -583,7 +584,7 @@ class Handler:
                 pass
             else:
                 if ca.artifact:
-                    return await self._serve_content_artifact(ca, headers)
+                    return await self._serve_content_artifact(ca, headers, request)
                 else:
                     return await self._stream_content_artifact(
                         request, StreamResponse(headers=headers), ca
@@ -597,7 +598,7 @@ class Handler:
             remote = await sync_to_async(cast_remote_blocking)()
 
             try:
-                url = remote.get_remote_artifact_url(rel_path)
+                url = remote.get_remote_artifact_url(rel_path, request=request)
 
                 def get_remote_artifact_blocking():
                     ra = RemoteArtifact.objects.select_related(
@@ -610,7 +611,7 @@ class Handler:
                 ra = await sync_to_async(get_remote_artifact_blocking)()
                 ca = ra.content_artifact
                 if ca.artifact:
-                    return await self._serve_content_artifact(ca, headers)
+                    return await self._serve_content_artifact(ca, headers, request)
                 else:
                     return await self._stream_content_artifact(
                         request, StreamResponse(headers=headers), ca
@@ -747,7 +748,7 @@ class Handler:
                 content_artifact.save()
         return artifact
 
-    async def _serve_content_artifact(self, content_artifact, headers):
+    async def _serve_content_artifact(self, content_artifact, headers, request):
         """
         Handle response for a Content Artifact with the file present.
 
@@ -758,6 +759,7 @@ class Handler:
             content_artifact (:class:`pulpcore.app.models.ContentArtifact`): The Content Artifact to
                 respond with.
             headers (dict): A dictionary of response headers.
+            request(:class:`~aiohttp.web.Request`): The request to prepare a response for.
 
         Raises:
             :class:`aiohttp.web_exceptions.HTTPFound`: When we need to redirect to the file
@@ -770,18 +772,28 @@ class Handler:
         artifact_name = artifact_file.name
 
         if settings.DEFAULT_FILE_STORAGE == "pulpcore.app.models.storage.FileSystem":
-            return FileResponse(os.path.join(settings.MEDIA_ROOT, artifact_name), headers=headers)
+            path = os.path.join(settings.MEDIA_ROOT, artifact_name)
+            if not os.path.exists(path):
+                raise Exception(_("Expected path '{}' is not found").format(path))
+            return FileResponse(path, headers=headers)
         elif settings.DEFAULT_FILE_STORAGE == "storages.backends.s3boto3.S3Boto3Storage":
             content_disposition = f"attachment;filename={content_artifact.relative_path}"
             parameters = {"ResponseContentDisposition": content_disposition}
             if headers.get("Content-Type"):
                 parameters["ResponseContentType"] = headers.get("Content-Type")
             url = URL(
-                artifact_file.storage.url(artifact_file.name, parameters=parameters), encoded=True
+                artifact_file.storage.url(
+                    artifact_file.name, parameters=parameters, http_method=request.method
+                ),
+                encoded=True,
             )
             raise HTTPFound(url)
         elif settings.DEFAULT_FILE_STORAGE == "storages.backends.azure_storage.AzureStorage":
-            url = URL(artifact_file.storage.url(artifact_name), encoded=True)
+            content_disposition = f"attachment;filename={artifact_name}"
+            parameters = {"content_disposition": content_disposition}
+            if headers.get("Content-Type"):
+                parameters["content_type"] = headers.get("Content-Type")
+            url = URL(artifact_file.storage.url(artifact_name, parameters=parameters), encoded=True)
             raise HTTPFound(url)
         else:
             raise NotImplementedError()
@@ -813,10 +825,9 @@ class Handler:
         async def handle_response_headers(headers):
             for name, value in headers.items():
                 lower_name = name.lower()
-                if lower_name in self.hop_by_hop_headers:
-                    continue
-
-                if response.status == 206 and lower_name == "content-length":
+                if lower_name not in self.hop_by_hop_headers:
+                    response.headers[name] = value
+                elif response.status == 206 and lower_name == "content-length":
                     content_length = int(value)
                     start = 0 if range_start is None else range_start
                     stop = content_length if range_stop is None else range_stop
@@ -829,9 +840,6 @@ class Handler:
                     response.headers["Content-Range"] = "bytes {0}-{1}/{2}".format(
                         start, stop - 1, content_length
                     )
-                    continue
-
-                response.headers[name] = value
             await response.prepare(request)
 
         data_size_handled = 0
