@@ -1,4 +1,4 @@
 from .base import BaseDownloader, DownloadResult  # noqa
 from .factory import DownloaderFactory  # noqa
 from .file import FileDownloader  # noqa
-from .http import http_giveup, HttpDownloader  # noqa
+from .http import HttpDownloader  # noqa
