@@ -14,10 +14,12 @@ from pulpcore.app.models import (  # noqa
     ContentArtifact,
     ContentManager,
     ContentGuard,
+    ContentRedirectContentGuard,
     CreatedResource,
     Distribution,
     Export,
     Exporter,
+    Group,
     GroupProgressReport,
     Import,
     Importer,
