@@ -6,10 +6,12 @@ from pulpcore.app.serializers import (  # noqa
     AsyncOperationResponseSerializer,
     ContentChecksumSerializer,
     ContentGuardSerializer,
+    ContentRedirectContentGuardSerializer,
     DetailRelatedField,
     DistributionSerializer,
     ExporterSerializer,
     ExportSerializer,
+    GetOrCreateSerializerMixin,
     IdentityField,
     ImporterSerializer,
     ImportSerializer,
@@ -28,6 +30,7 @@ from pulpcore.app.serializers import (  # noqa
     SingleArtifactContentSerializer,
     SingleContentArtifactField,
     TaskGroupOperationResponseSerializer,
+    RepositoryAddRemoveContentSerializer,
     ValidateFieldsMixin,
     validate_unknown_fields,
 )
