@@ -3,17 +3,25 @@ from collections import defaultdict
 from gettext import gettext as _
 import logging
 
-from django.db.models import Prefetch, prefetch_related_objects
+from asgiref.sync import sync_to_async
+from django.db.models import Prefetch, prefetch_related_objects, Q
 
 from pulpcore.plugin.exceptions import UnsupportedDigestValidationError
-from pulpcore.plugin.models import Artifact, ContentArtifact, ProgressReport, RemoteArtifact
+from pulpcore.plugin.models import (
+    AlternateContentSource,
+    Artifact,
+    ContentArtifact,
+    ProgressReport,
+    RemoteArtifact,
+)
+from pulpcore.plugin.sync import sync_to_async_iterable
 
 from .api import Stage
 
 log = logging.getLogger(__name__)
 
 
-def _check_for_forbidden_checksume_type(artifact):
+def _check_for_forbidden_checksum_type(artifact):
     """Check if content doesn't have forbidden checksum type.
 
     If contains forbidden checksum type it will raise ValueError,
@@ -72,7 +80,7 @@ class QueryExistingArtifacts(Stage):
                 for d_artifact in d_content.d_artifacts:
                     if d_artifact.artifact._state.adding:
                         if not d_artifact.deferred_download:
-                            _check_for_forbidden_checksume_type(d_artifact.artifact)
+                            _check_for_forbidden_checksum_type(d_artifact.artifact)
                         for digest_type in Artifact.COMMON_DIGEST_FIELDS:
                             digest_value = getattr(d_artifact.artifact, digest_type)
                             if digest_value:
@@ -85,13 +93,14 @@ class QueryExistingArtifacts(Stage):
             # swap it out with the existing one.
             for digest_type, digests in artifact_digests_by_type.items():
                 query_params = {"{attr}__in".format(attr=digest_type): digests}
-                existing_artifacts = Artifact.objects.filter(**query_params).only(digest_type)
-                existing_artifacts.touch()
+                existing_artifacts_qs = Artifact.objects.filter(**query_params)
+                existing_artifacts = sync_to_async_iterable(existing_artifacts_qs)
+                await sync_to_async(existing_artifacts_qs.touch)()
                 for d_content in batch:
                     for d_artifact in d_content.d_artifacts:
                         artifact_digest = getattr(d_artifact.artifact, digest_type)
                         if artifact_digest:
-                            for result in existing_artifacts:
+                            async for result in existing_artifacts:
                                 result_digest = getattr(result, digest_type)
                                 if result_digest == artifact_digest:
                                     d_artifact.artifact = result
@@ -156,7 +165,7 @@ class ArtifactDownloader(Stage):
         #    Set to None if stage is shutdown.
         content_get_task = _add_to_pending(content_iterator.__anext__())
 
-        with ProgressReport(
+        async with ProgressReport(
             message="Downloading Artifacts", code="sync.downloading.artifacts"
         ) as pb:
             try:
@@ -172,7 +181,7 @@ class ArtifactDownloader(Stage):
                                 content_get_task = None
                         else:
                             pb.done += task.result()  # download_count
-                            pb.save()
+                            await pb.asave()
 
                     if content_get_task and content_get_task not in pending:  # not yet shutdown
                         if len(pending) < self.max_concurrent_content:
@@ -233,12 +242,13 @@ class ArtifactSaver(Stage):
                     if d_artifact.artifact._state.adding and not d_artifact.deferred_download:
                         d_artifact.artifact.file = str(d_artifact.artifact.file)
                         da_to_save.append(d_artifact)
+            da_to_save_ordered = sorted(da_to_save, key=lambda x: x.artifact.sha256)
 
             if da_to_save:
                 for d_artifact, artifact in zip(
-                    da_to_save,
-                    Artifact.objects.bulk_get_or_create(
-                        d_artifact.artifact for d_artifact in da_to_save
+                    da_to_save_ordered,
+                    await sync_to_async(Artifact.objects.bulk_get_or_create)(
+                        d_artifact.artifact for d_artifact in da_to_save_ordered
                     ),
                 ):
                     d_artifact.artifact = artifact
@@ -267,11 +277,11 @@ class RemoteArtifactSaver(Stage):
             The coroutine for this stage.
         """
         async for batch in self.batches():
-            self._handle_remote_artifacts(batch)
+            await self._handle_remote_artifacts(batch)
             for d_content in batch:
                 await self.put(d_content)
 
-    def _handle_remote_artifacts(self, batch):
+    async def _handle_remote_artifacts(self, batch):
         """
         Build a list of only :class:`~pulpcore.plugin.models.RemoteArtifact` that need
         to be created for the batch.
@@ -284,17 +294,16 @@ class RemoteArtifactSaver(Stage):
         """
         remotes_present = set()
         for d_content in batch:
-            # If the attribute is set in a previous batch on the very first item in this batch, the
-            # rest of the items in this batch will not get the attribute set during prefetch.
-            # https://code.djangoproject.com/ticket/32089
+            # https://code.djangoproject.com/ticket/33596
+            # If the content was pre-fetched previously, remove that cached data, which could be out
+            # of date.
             if hasattr(d_content.content, "_remote_artifact_saver_cas"):
                 delattr(d_content.content, "_remote_artifact_saver_cas")
-
             for d_artifact in d_content.d_artifacts:
                 if d_artifact.remote:
                     remotes_present.add(d_artifact.remote)
 
-        prefetch_related_objects(
+        await sync_to_async(prefetch_related_objects)(
             [d_c.content for d_c in batch],
             Prefetch(
                 "contentartifact_set",
@@ -320,7 +329,9 @@ class RemoteArtifactSaver(Stage):
                 if not d_artifact.remote:
                     continue
 
-                for content_artifact in d_content.content._remote_artifact_saver_cas:
+                async for content_artifact in sync_to_async_iterable(
+                    d_content.content._remote_artifact_saver_cas
+                ):
                     if d_artifact.relative_path == content_artifact.relative_path:
                         break
                 else:
@@ -370,7 +381,9 @@ class RemoteArtifactSaver(Stage):
                             msg.format(rp=d_artifact.relative_path, c=d_content.content)
                         )
 
-                for remote_artifact in content_artifact._remote_artifact_saver_ras:
+                async for remote_artifact in sync_to_async_iterable(
+                    content_artifact._remote_artifact_saver_ras
+                ):
                     if d_artifact.url == remote_artifact.url:
                         break
 
@@ -389,10 +402,12 @@ class RemoteArtifactSaver(Stage):
         # Artifact sha256 for our ordering.
         if ras_to_create:
             ras_to_create_ordered = sorted(list(ras_to_create.values()), key=lambda x: x.sha256)
-            RemoteArtifact.objects.bulk_create(ras_to_create_ordered)
+            await sync_to_async(RemoteArtifact.objects.bulk_create)(ras_to_create_ordered)
         if ras_to_update:
             ras_to_update_ordered = sorted(list(ras_to_update.values()), key=lambda x: x.sha256)
-            RemoteArtifact.objects.bulk_update(ras_to_update_ordered, fields=["url"])
+            await sync_to_async(RemoteArtifact.objects.bulk_update)(
+                ras_to_update_ordered, fields=["url"]
+            )
 
     @staticmethod
     def _create_remote_artifact(d_artifact, content_artifact):
@@ -410,3 +425,60 @@ class RemoteArtifactSaver(Stage):
         )
         ra.validate_checksums()
         return ra
+
+
+class ACSArtifactHandler(Stage):
+    """
+    API stage to download :class:`~pulpcore.plugin.models.Artifact` files from Alternate
+    Content Source if available.
+    """
+
+    async def run(self):
+        async for batch in self.batches():
+            acs_exists = await sync_to_async(AlternateContentSource.objects.exists)()
+            if acs_exists:
+                # Gather batch d_artifact checksums
+                batch_checksums = defaultdict(list)
+                for d_content in batch:
+                    for d_artifact in d_content.d_artifacts:
+                        for cks_type in d_artifact.artifact.COMMON_DIGEST_FIELDS:
+                            if getattr(d_artifact.artifact, cks_type):
+                                batch_checksums[cks_type].append(
+                                    getattr(d_artifact.artifact, cks_type)
+                                )
+
+                batch_query = Q()
+                for checksum_type in batch_checksums.keys():
+                    batch_query.add(
+                        Q(**{f"{checksum_type}__in": batch_checksums[checksum_type]}), Q.OR
+                    )
+
+                existing_ras = await sync_to_async(list)(
+                    (
+                        RemoteArtifact.objects.acs()
+                        .filter(batch_query)
+                        .only("url", "remote")
+                        .select_related("remote")
+                    )
+                )
+                existing_ras_dict = dict()
+                for ra in existing_ras:
+                    for c_type in Artifact.COMMON_DIGEST_FIELDS:
+                        checksum = await sync_to_async(getattr)(ra, c_type)
+                        if checksum:
+                            existing_ras_dict[checksum] = {
+                                "remote": ra.remote,
+                                "url": ra.url,
+                            }
+
+                for d_content in batch:
+                    for d_artifact in d_content.d_artifacts:
+                        for checksum_type in Artifact.COMMON_DIGEST_FIELDS:
+                            if getattr(d_artifact.artifact, checksum_type):
+                                checksum = getattr(d_artifact.artifact, checksum_type)
+                                if checksum in existing_ras_dict:
+                                    d_artifact.url = existing_ras_dict[checksum]["url"]
+                                    d_artifact.remote = existing_ras_dict[checksum]["remote"]
+
+            for d_content in batch:
+                await self.put(d_content)
