@@ -2,7 +2,6 @@ from gettext import gettext as _
 
 import asyncio
 import importlib
-import json
 import logging
 import os
 import random
@@ -24,7 +23,6 @@ django.setup()
 from django.conf import settings  # noqa: E402: module level not at top of file
 from django.db import connection  # noqa: E402: module level not at top of file
 from django.utils import timezone  # noqa: E402: module level not at top of file
-from guardian.shortcuts import get_users_with_perms  # noqa: E402: module level not at top of file
 from django_currentuser.middleware import (  # noqa: E402: module level not at top of file
     _set_current_user,
 )
@@ -32,6 +30,10 @@ from django_guid import set_guid  # noqa: E402: module level not at top of file
 
 from pulpcore.app.models import Worker, Task  # noqa: E402: module level not at top of file
 
+from pulpcore.app.role_util import (  # noqa: E402: module level not at top of file
+    get_users_with_perms,
+)
+
 from pulpcore.constants import (  # noqa: E402: module level not at top of file
     TASK_STATES,
     TASK_INCOMPLETE_STATES,
@@ -39,6 +41,7 @@ from pulpcore.constants import (  # noqa: E402: module level not at top of file
 
 from pulpcore.exceptions import AdvisoryLockError  # noqa: E402: module level not at top of file
 from pulpcore.tasking.storage import WorkerDirectory  # noqa: E402: module level not at top of file
+from pulpcore.tasking.tasks import dispatch_scheduled_tasks  # noqa: E402
 from pulpcore.tasking.util import _delete_incomplete_resources  # noqa: E402
 
 
@@ -49,6 +52,27 @@ TASK_GRACE_INTERVAL = 3
 WORKER_CLEANUP_INTERVAL = 100
 
 
+class PGAdvisoryLock:
+    def __init__(self, lock, lock_group=0):
+        self.lock_group = lock_group
+        self.lock = lock
+
+    def __enter__(self):
+        with connection.cursor() as cursor:
+            cursor.execute("SELECT pg_try_advisory_lock(%s, %s);", [self.lock_group, self.lock])
+            acquired = cursor.fetchone()[0]
+        if not acquired:
+            raise AdvisoryLockError("Could not acquire lock.")
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        with connection.cursor() as cursor:
+            cursor.execute("SELECT pg_advisory_unlock(%s, %s);", [self.lock_group, self.lock])
+            released = cursor.fetchone()[0]
+        if not released:
+            raise RuntimeError("Lock not held.")
+
+
 def handle_worker_heartbeat(worker_name):
     """
     This is a generic function for updating worker heartbeat records.
@@ -110,11 +134,11 @@ class NewPulpWorker:
         _logger.info(_("Worker %s was shut down."), self.name)
 
     def worker_cleanup(self):
-        qs = Worker.objects.missing_workers()
+        qs = Worker.objects.missing_workers(age=timedelta(days=7))
         if qs:
             for worker in qs:
                 _logger.info(_("Clean missing worker %s."), worker.name)
-                worker.delete()
+            qs.delete()
 
     def beat(self):
         if self.worker.last_heartbeat < timezone.now() - timedelta(seconds=self.heartbeat_period):
@@ -125,6 +149,8 @@ class NewPulpWorker:
             if self.worker_cleanup_countdown <= 0:
                 self.worker_cleanup_countdown = WORKER_CLEANUP_INTERVAL
                 self.worker_cleanup()
+            with suppress(AdvisoryLockError), PGAdvisoryLock(42):
+                dispatch_scheduled_tasks()
 
     def notify_workers(self):
         self.cursor.execute("NOTIFY pulp_worker_wakeup")
@@ -207,35 +233,24 @@ class NewPulpWorker:
                             # without considering this tasks resources
                             # as we just released them
                             continue
-                    if settings.ALLOW_SHARED_TASK_RESOURCES:
-                        # This statement is using lazy evaluation
-                        if (
-                            task.state == TASK_STATES.WAITING
-                            # No exclusive resource taken?
-                            and not any(
-                                resource in taken_exclusive_resources
-                                or resource in taken_shared_resources
-                                for resource in exclusive_resources
-                            )
-                            # No shared resource exclusively taken?
-                            and not any(
-                                resource in taken_exclusive_resources
-                                for resource in shared_resources
-                            )
-                        ):
-                            yield task
-                            # Start from the top of the Task list
-                            break
-                    else:
-                        # Treat all resources exclusively
-                        if task.state == TASK_STATES.WAITING and not any(
+                    # This statement is using lazy evaluation
+                    if (
+                        task.state == TASK_STATES.WAITING
+                        # No exclusive resource taken?
+                        and not any(
                             resource in taken_exclusive_resources
                             or resource in taken_shared_resources
-                            for resource in exclusive_resources + shared_resources
-                        ):
-                            yield task
-                            # Start from the top of the Task list
-                            break
+                            for resource in exclusive_resources
+                        )
+                        # No shared resource exclusively taken?
+                        and not any(
+                            resource in taken_exclusive_resources for resource in shared_resources
+                        )
+                    ):
+                        yield task
+                        # Start from the top of the Task list
+                        break
+
                 # Record the resources of the pending task we didn't get
                 taken_exclusive_resources.update(exclusive_resources)
                 taken_shared_resources.update(shared_resources)
@@ -319,6 +334,21 @@ class NewPulpWorker:
                         cancel_reason = "Aborted during worker shutdown."
                         break
             task_process.join()
+            if not cancel_state and task_process.exitcode != 0:
+                _logger.warn(
+                    _("Task process for %s exited with non zero exitcode %i."),
+                    task.pk,
+                    task_process.exitcode,
+                )
+                cancel_state = TASK_STATES.FAILED
+                if task_process.exitcode < 0:
+                    cancel_reason = "Killed by signal {sig_num}.".format(
+                        sig_num=-task_process.exitcode
+                    )
+                else:
+                    cancel_reason = "Task process died unexpectedly with exitcode {code}.".format(
+                        code=task_process.exitcode
+                    )
             if cancel_state:
                 self.cancel_abandoned_task(task, cancel_state, cancel_reason)
         if task.reserved_resources_record:
@@ -364,7 +394,7 @@ def _perform_task(task_pk, task_working_dir_rel_path):
     task.set_running()
     # Store the task id in the environment for `Task.current()`.
     os.environ["PULP_TASK_ID"] = str(task.pk)
-    user = get_users_with_perms(task).first()
+    user = get_users_with_perms(task, with_group_users=False).first()
     _set_current_user(user)
     set_guid(task.logging_cid)
     try:
@@ -374,8 +404,8 @@ def _perform_task(task_pk, task_working_dir_rel_path):
         module_name, function_name = task.name.rsplit(".", 1)
         module = importlib.import_module(module_name)
         func = getattr(module, function_name)
-        args = json.loads(task.args) or ()
-        kwargs = json.loads(task.kwargs) or {}
+        args = task.args or ()
+        kwargs = task.kwargs or {}
         os.chdir(task_working_dir_rel_path)
         result = func(*args, **kwargs)
         if asyncio.iscoroutine(result):
