@@ -1,14 +1,13 @@
-import json
 import logging
-import uuid
 from gettext import gettext as _
 
 from django.db import transaction, connection
-from django.db.models import Model
+from django.db.models import Model, Q
+from django.utils import timezone
 from django_guid import get_guid
 
-from pulpcore.app.models import Task
-from pulpcore.constants import TASK_STATES
+from pulpcore.app.models import Task, TaskSchedule
+from pulpcore.constants import TASK_FINAL_STATES, TASK_STATES
 from pulpcore.tasking import util
 
 _logger = logging.getLogger(__name__)
@@ -17,13 +16,6 @@ _logger = logging.getLogger(__name__)
 TASK_TIMEOUT = -1  # -1 for infinite timeout
 
 
-class UUIDEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, uuid.UUID):
-            return str(obj)
-        return super().default(obj)
-
-
 def _validate_and_get_resources(resources):
     resource_set = set()
     for r in resources:
@@ -36,6 +28,12 @@ def _validate_and_get_resources(resources):
     return list(resource_set)
 
 
+def _wakeup_worker():
+    # Notify workers
+    with connection.connection.cursor() as cursor:
+        cursor.execute("NOTIFY pulp_worker_wakeup")
+
+
 def dispatch(
     func,
     args=None,
@@ -57,7 +55,7 @@ def dispatch(
     ``uuid.UUID``.
 
     Args:
-        func (callable): The function to be run by RQ when the necessary locks are acquired.
+        func (callable | str): The function to be run when the necessary locks are acquired.
         args (tuple): The positional arguments to pass on to the task.
         kwargs (dict): The keyword arguments to pass on to the task.
         task_group (pulpcore.app.models.TaskGroup): A TaskGroup to add the created Task to.
@@ -72,11 +70,12 @@ def dispatch(
         ValueError: When `resources` is an unsupported type.
     """
 
+    if callable(func):
+        func = f"{func.__module__}.{func.__name__}"
+
     if exclusive_resources is None:
         exclusive_resources = []
 
-    args_as_json = json.dumps(args, cls=UUIDEncoder)
-    kwargs_as_json = json.dumps(kwargs, cls=UUIDEncoder)
     resources = _validate_and_get_resources(exclusive_resources)
     if shared_resources:
         resources.extend(
@@ -87,13 +86,47 @@ def dispatch(
             state=TASK_STATES.WAITING,
             logging_cid=(get_guid() or ""),
             task_group=task_group,
-            name=f"{func.__module__}.{func.__name__}",
-            args=args_as_json,
-            kwargs=kwargs_as_json,
+            name=func,
+            args=args,
+            kwargs=kwargs,
             parent_task=Task.current(),
             reserved_resources_record=resources,
         )
-    # Notify workers
-    with connection.connection.cursor() as cursor:
-        cursor.execute("NOTIFY pulp_worker_wakeup")
+        transaction.on_commit(_wakeup_worker)
+
     return task
+
+
+def dispatch_scheduled_tasks():
+    # Warning, dispatch_scheduled_tasks is not race condition free!
+    now = timezone.now()
+    # Dispatch all tasks old enough and not still running
+    for task_schedule in TaskSchedule.objects.filter(next_dispatch__lte=now).filter(
+        Q(last_task=None) | Q(last_task__state__in=TASK_FINAL_STATES)
+    ):
+        try:
+            if task_schedule.dispatch_interval is None:
+                # This was a timed one shot task schedule
+                task_schedule.next_dispatch = None
+            else:
+                # This is a recurring task schedule
+                while task_schedule.next_dispatch < now:
+                    # Do not schedule in the past
+                    task_schedule.next_dispatch += task_schedule.dispatch_interval
+            with transaction.atomic():
+                task_schedule.last_task = dispatch(
+                    task_schedule.task_name,
+                )
+                task_schedule.save(update_fields=["next_dispatch", "last_task"])
+
+            _logger.info(
+                _("Dispatched scheduled task {task_name} as task id {task_id}").format(
+                    task_name=task_schedule.task_name, task_id=task_schedule.last_task.pk
+                )
+            )
+        except Exception as e:
+            _logger.warn(
+                _("Dispatching scheduled task {task_name} failed. {error}").format(
+                    task_name=task_schedule.task_name, error=str(e)
+                )
+            )
