@@ -11,6 +11,7 @@ from pulp_smash.pulp3.bindings import monitor_task
 from pulp_smash.pulp3.utils import gen_repo
 
 from requests.exceptions import HTTPError
+import pytest
 
 from pulpcore.tests.functional.api.using_plugin.utils import gen_file_remote
 from pulpcore.tests.functional.api.using_plugin.constants import (
@@ -266,7 +267,7 @@ class CRUDRemoteTestCase(unittest.TestCase):
         cli_client = cli.Client(self.cfg)
         remote = self.remotes_api.create({"name": "test_pass", "url": "http://", "password": "new"})
         href = remote.pulp_href
-        uuid = re.search(r"/pulp/api/v3/remotes/file/file/([\w-]+)/", href).group(1)
+        uuid = re.search(r"/api/v3/remotes/file/file/([\w-]+)/", href).group(1)
         shell_cmd = (
             f"import pulpcore; print(pulpcore.app.models.Remote.objects.get(pk='{uuid}').password)"
         )
@@ -285,7 +286,7 @@ class CRUDRemoteTestCase(unittest.TestCase):
         cli_client = cli.Client(self.cfg)
         remote = self.remotes_api.create({"name": "test_pass", "url": "http://", "password": "new"})
         href = remote.pulp_href
-        uuid = re.search(r"/pulp/api/v3/remotes/file/file/([\w-]+)/", href).group(1)
+        uuid = re.search(r"/api/v3/remotes/file/file/([\w-]+)/", href).group(1)
         shell_cmd = (
             f"import pulpcore; print(pulpcore.app.models.Remote.objects.get(pk='{uuid}').password)"
         )
@@ -360,6 +361,7 @@ class CRUDRemoteTestCase(unittest.TestCase):
         self.remotes_api.partial_update(self.remote.pulp_href, data)
 
 
+@pytest.mark.parallel
 class CreatePulpLabelsRemoteTestCase(unittest.TestCase):
     """A test case for verifying whether pulp_labels are correctly assigned to a new remote."""
 
@@ -398,6 +400,7 @@ class CreatePulpLabelsRemoteTestCase(unittest.TestCase):
         self.assertEqual(remote["pulp_labels"], self.pulp_labels)
 
 
+@pytest.mark.parallel
 class RemoteFileURLsValidationTestCase(unittest.TestCase):
     """A test case that verifies the validation of remotes' URLs."""
 
@@ -443,3 +446,17 @@ class RemoteFileURLsValidationTestCase(unittest.TestCase):
 
         remote = self.remotes_api.create(remote_attrs)
         self.addCleanup(self.remotes_api.delete, remote.pulp_href)
+
+    def test_no_username_password(self):
+        """Test that the remote url can't contain username/password."""
+        remote_attrs = {
+            "name": utils.uuid4(),
+            "url": "http://elladan@rivendell.org",
+        }
+        self.raise_for_invalid_request(remote_attrs)
+
+        remote_attrs = {
+            "name": utils.uuid4(),
+            "url": "http://elladan:pass@rivendell.org",
+        }
+        self.raise_for_invalid_request(remote_attrs)
