@@ -1,7 +1,6 @@
 """Tests that perform actions over distributions."""
 import csv
 import hashlib
-import os
 from time import sleep
 import unittest
 from urllib.parse import urljoin
@@ -21,6 +20,8 @@ from pulp_smash.pulp3.utils import (
 )
 from requests.exceptions import HTTPError
 
+from pulpcore.client.pulpcore import ApiException, StatusApi
+
 from pulpcore.client.pulp_file import (
     ContentFilesApi,
     DistributionsFileApi,
@@ -43,6 +44,7 @@ from pulpcore.tests.functional.api.using_plugin.utils import (
     create_file_publication,
     gen_file_remote,
     gen_file_client,
+    gen_pulpcore_client,
     monitor_task,
 )
 from pulpcore.tests.functional.api.using_plugin.utils import set_up_module as setUpModule  # noqa
@@ -386,10 +388,6 @@ class ContentServePublicationDistributionTestCase(unittest.TestCase):
 
     def test_content_served_immediate_with_range_request_start_value_larger_than_content(self):
         """Assert that a range request with a start value larger than the content errors."""
-        if "azure" == os.getenv("TEST"):
-            # Skip while Azurite is not handling this correctly:
-            # https://github.com/Azure/Azurite/issues/333
-            self.skipTest("Skipping test for Azurite storage.")
         self.setup_download_test("immediate", url=FILE_CHUNKED_FIXTURE_MANIFEST_URL)
         with self.assertRaises(HTTPError) as cm:
             download_content_unit_return_requests_response(
@@ -418,8 +416,16 @@ class ContentServePublicationDistributionTestCase(unittest.TestCase):
             postgresql_found, "PostgreSQL service not found or is not active. Can't restart it."
         )
         svc_mgr.restart([postgresql_service_name])
-        # Wait for postgres to come back and all services to recover
-        sleep(2)
+        # Wait for postgres to come back and pulpcore-api to recover
+        status_api = StatusApi(gen_pulpcore_client())
+        for i in range(5):
+            sleep(2)
+            try:
+                status_api.status_read()
+                break
+            except ApiException:
+                if i == 4:
+                    raise
         self.setup_download_test("immediate")
         self.do_test_content_served()
         url_fragments = [
