@@ -1,4 +1,5 @@
 import asyncio
+from unittest import skip
 from unittest.mock import patch, call
 
 from django.db.utils import InterfaceError, OperationalError
@@ -8,6 +9,7 @@ from pulpcore.content import _heartbeat
 
 
 class ContentHeartbeatTestCase(TestCase):
+    @skip("Skipping while resolving https://github.com/rochacbruno/dynaconf/issues/689")
     @patch("pulpcore.app.models.ContentAppStatus.objects.get_or_create")
     @patch("pulpcore.content.handler.Handler._reset_db_connection")
     def test_db_connection_interface_error(self, mock_reset_db, mock_get_or_create):
